﻿/* Copyright 2019, Kasyan Servetsky
October 27, 2019
Written by Kasyan Servetsky
http://www.kasyan.ho.com.ua
e-mail: askoldich@yahoo.com */
//======================================================================================
var scriptName = "Export to PDF - 2.4",
debugMode = false, // for debugging only
count = 0,
outputFolder = null,
progressWin, progressBar, progressTxt, // Progress bar
doc, set, dataMerge, dataMergePrefs, logFolderPath, pdfExportPresetOnline, pdfExportPresetPrint, baseName;

PreCheck();
//===================================== FUNCTIONS ======================================
function Main() {
	try {
		var startTime = new Date(),
		x, o,
		printArr = [],
		onlineArr = [],
		nopricesArr = [];

		baseName = doc.name.replace(/\.indd$/, ""); // global

		dataMergePrefs.recordSelection = RecordSelection.ONE_RECORD;

		// Log folder
		if (set.outputOversetReportFile) {
			logFolderPath = outputFolder + "/Logs/"; // global
			var folderLog = new Folder(logFolderPath);
			if (!folderLog.exists) folderLog.create();
		}
	
		// Export
		for (x in set) {
			o = eval("set." + x);

			if (o instanceof Object) {
				if ((set.languageRb == 0 && o.lang != "HU") || (set.languageRb == 1 && o.lang == "HU")) {
					if (o.cbPrint) printArr.push(o.lang);
					if (o.cbOnline) onlineArr.push(o.lang);
					if (o.cbNoprices) nopricesArr.push(o.lang);
				}
			}
		}

		// Progress bar
		var totalPdfsCount = printArr.length + onlineArr.length + nopricesArr.length;
		
		progressWin = new Window("window", scriptName);
		progressBar = progressWin.add("progressbar" , undefined, 0, totalPdfsCount);
		progressBar.preferredSize.width = 600;
		progressTxt = progressWin.add("statictext", undefined,  "");
		progressTxt.alignment = "fill";
		progressWin.show();

		app.scriptPreferences.userInteractionLevel = UserInteractionLevels.NEVER_INTERACT;

		if (printArr.length > 0) {
			ExportPdf(printArr, "Print");
		}

		if (onlineArr.length > 0) {
			ExportPdf(onlineArr, "Online");
		}

		if (nopricesArr.length > 0) {
			ExportPdf(nopricesArr, "NoPrices");
			doc.revert();
		}
		
		app.scriptPreferences.userInteractionLevel = UserInteractionLevels.INTERACT_WITH_ALL;
		
		progressWin.close();
		var endTime = new Date();
		var duration = GetDuration(startTime, endTime);
		var report = count + " PDF-file" + ((count == 1) ? " was" : "s were") + " exported.\n(time elapsed: " + duration + ")";
		
		if (debugMode) {
			$.writeln("==================\r" + report);
		}
		else {
			alert("Finished. " + report, scriptName);
		}

	}
	catch(err) {
		app.scriptPreferences.userInteractionLevel = UserInteractionLevels.INTERACT_WITH_ALL;
		
		if (debugMode) {
			$.writeln(err.message + ", line: " + err.line);
		}
		else {
			ErrorExit("ERROR: " + err.message + ", line: " + err.line, scriptName, true);
		}
	}
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function ExportPdf(arr, mode) {
	try {
		if (debugMode) $.writeln("mode = " + mode);

		var pdfName, filePdf, pdfExportPreset, lang, recordNumber,
		outputOversetReportFile, // leave it 'undefined' so it wouldn't produce 'an illegal parameter' error on export
		swatchNone = doc.swatches.itemByName("None");
		
		// PDF preset
		if (mode == "Print") {
			pdfExportPreset = pdfExportPresetPrint;
		}
		else if (mode == "Online") {
			pdfExportPreset = pdfExportPresetOnline;
		}
		else if (mode == "NoPrices") {
			pdfExportPreset = pdfExportPresetOnline;
			
			// Hide layers
			var cenowkiLayer = doc.layers.itemByName("Cenówki");
			if (cenowkiLayer.isValid) cenowkiLayer.visible = false;

			var skresleniaLayer = doc.layers.itemByName("skreślenia");
			if (skresleniaLayer.isValid) skresleniaLayer.visible = false;

			// Change the "character color" to NONE in the following styles
			var tabelaParStGr = doc.paragraphStyleGroups.itemByName("Tabela");
			if (tabelaParStGr.isValid) {
				var tabelaCenaParSt  = tabelaParStGr.paragraphStyles.itemByName("Tabela cena");
				if (tabelaCenaParSt.isValid) tabelaCenaParSt.fillColor = swatchNone;

				var tabelaCenaPromoParSt  = tabelaParStGr.paragraphStyles.itemByName("Tabela cena promo");
				if (tabelaCenaPromoParSt.isValid) tabelaCenaPromoParSt.fillColor = swatchNone;
			}

			var pkCenaCzarnaCharSt= doc.characterStyles.itemByName("PK cena czarna");
			if (pkCenaCzarnaCharSt.isValid) pkCenaCzarnaCharSt.fillColor = swatchNone;

			var pkCenaCzerwonaCharSt = doc.characterStyles.itemByName("PK cena czerwona");
			if (pkCenaCzerwonaCharSt.isValid) pkCenaCzerwonaCharSt.fillColor = swatchNone;

			var komplementarnyCenaPodstawowaCharSt= doc.characterStyles.itemByName("Komplementarny cena podstawowa");
			if (komplementarnyCenaPodstawowaCharSt.isValid) komplementarnyCenaPodstawowaCharSt.fillColor = swatchNone;

			var komplementarnyCenaPromocyjnaCharSt= doc.characterStyles.itemByName("Komplementarny cena promocyjna");
			if (komplementarnyCenaPromocyjnaCharSt.isValid) komplementarnyCenaPromocyjnaCharSt.fillColor = swatchNone;
		}
	
		for (var i = 0; i < arr.length; i++) {
			lang = arr[i];

			switch (lang) {
				case "CZ":
					recordNumber = 1;
					break;
				case "EN":
					recordNumber = 2;
					break;
				case "LT":
					recordNumber = 4;
					break;
				case "LV":
					recordNumber = 5;
					break;
				case "RO":
					recordNumber = 6;
					break;
				case "SK":
					recordNumber = 7;
					break;
				case "HU":
					recordNumber = 3;
					break;
				default:
					ErrorExit("Unexpected error while selecting a record number.", true);
			}

			dataMergePrefs.recordNumber = recordNumber;

			pdfName = baseName + "_" + lang + "_" + mode + ".pdf";
			filePdf = new File(set.outputFolderPath + "/" + pdfName);
			
			count++;
			progressBar.value = count;
			progressTxt.text = pdfName;
			
			if (!filePdf.exists) {
				if (set.languageRb == 0 && set.outputOversetReportFile) outputOversetReportFile = new File(logFolderPath + pdfName.replace(/pdf$/, "txt"));
				
				if (!filePdf.exists && lang != "HU") {
					dataMerge.exportFile(filePdf, pdfExportPreset, outputOversetReportFile);
				}
				else {
					doc.exportFile(ExportFormat.PDF_TYPE, filePdf, false, pdfExportPreset);					
				}
			}
		}
	}
	catch(err) {
		app.scriptPreferences.userInteractionLevel = UserInteractionLevels.INTERACT_WITH_ALL;
		
		if (debugMode) {
			$.writeln(err.message + ", line: " + err.line);
		}
		else {
			alert("ERROR: " + err.message + ", line: " + err.line, scriptName, true);
		}
	}
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function CreateDialog() {
	var ddlWidth = 120;
	GetDialogSettings();
	var w = new Window("dialog", scriptName);
	w.orientation = "column";
	var container = w.add("group");
	container.orientation = "row";
	container.alignChildren = "top";	
	var lc = container.add("group");
	lc.orientation = "column";
	var rc = container.add("group");
	rc.orientation = "column";
	
	// Left column ==========================================================================
	lc.p = lc.add("panel", undefined, "Czech:");
	lc.p.orientation = "row";
	lc.p.alignment = "fill";
	lc.p.alignChildren = "left";
	lc.p.cb = lc.p.add("checkbox", undefined, "Print");
	lc.p.cb.value = set.czech.cbPrint;
	lc.p.cb1 = lc.p.add("checkbox", undefined, "Online");
	lc.p.cb1.value = set.czech.cbOnline;	
	lc.p.cb2 = lc.p.add("checkbox", undefined, "No prices");
	lc.p.cb2.value = set.czech.cbNoprices;	
	
	lc.p1 = lc.add("panel", undefined, "English:");
	lc.p1.orientation = "row";
	lc.p1.alignment = "fill";
	lc.p1.alignChildren = "left";
	lc.p1.cb = lc.p1.add("checkbox", undefined, "Print");
	lc.p1.cb.value = set.english.cbPrint;
	lc.p1.cb1 = lc.p1.add("checkbox", undefined, "Online");
	lc.p1.cb1.value = set.english.cbOnline;	
	lc.p1.cb2 = lc.p1.add("checkbox", undefined, "No prices");
	lc.p1.cb2.value = set.english.cbNoprices;		
	
	lc.p2 = lc.add("panel", undefined, "Lithuanian:");
	lc.p2.orientation = "row";
	lc.p2.alignment = "fill";
	lc.p2.alignChildren = "left";
	lc.p2.cb = lc.p2.add("checkbox", undefined, "Print");
	lc.p2.cb.value = set.lithuanian.cbPrint;
	lc.p2.cb1 = lc.p2.add("checkbox", undefined, "Online");
	lc.p2.cb1.value = set.lithuanian.cbOnline;	
	lc.p2.cb2 = lc.p2.add("checkbox", undefined, "No prices");
	lc.p2.cb2.value = set.lithuanian.cbNoprices;	
		
	lc.p3 = lc.add("panel", undefined, "Latvian:");
	lc.p3.orientation = "row";
	lc.p3.alignment = "fill";
	lc.p3.alignChildren = "left";
	lc.p3.cb = lc.p3.add("checkbox", undefined, "Print");
	lc.p3.cb.value = set.latvian.cbPrint;
	lc.p3.cb1 = lc.p3.add("checkbox", undefined, "Online");
	lc.p3.cb1.value = set.latvian.cbOnline;
	lc.p3.cb2 = lc.p3.add("checkbox", undefined, "No prices");
	lc.p3.cb2.value = set.latvian.cbNoprices;	
		
	lc.p4 = lc.add("panel", undefined, "Romanian:");
	lc.p4.orientation = "row";
	lc.p4.alignment = "fill";
	lc.p4.alignChildren = "left";
	lc.p4.cb = lc.p4.add("checkbox", undefined, "Print");
	lc.p4.cb.value = set.romanian.cbPrint;
	lc.p4.cb1 = lc.p4.add("checkbox", undefined, "Online");
	lc.p4.cb1.value = set.romanian.cbOnline;	
	lc.p4.cb2 = lc.p4.add("checkbox", undefined, "No prices");
	lc.p4.cb2.value = set.romanian.cbNoprices;	
		
	lc.p5 = lc.add("panel", undefined, "Slovakian:");
	lc.p5.orientation = "row";
	lc.p5.alignment = "fill";
	lc.p5.alignChildren = "left";
	lc.p5.cb = lc.p5.add("checkbox", undefined, "Print");
	lc.p5.cb.value = set.slovakian.cbPrint;
	lc.p5.cb1 = lc.p5.add("checkbox", undefined, "Online");
	lc.p5.cb1.value = set.slovakian.cbOnline;	
	lc.p5.cb2 = lc.p5.add("checkbox", undefined, "No prices");
	lc.p5.cb2.value = set.slovakian.cbNoprices;

	// Right column =========================================================================
	// Radio buttons: Hungarian / Main languages"
	rc.p = rc.add("panel", undefined, "Make:");
	rc.p.orientation = "column";
	rc.p.alignment = "fill";
	rc.p.alignChildren = "left";

	rc.p.rb = rc.p.add("radiobutton", undefined, "Main languages");
	rc.p.rb1 = rc.p.add("radiobutton", undefined, "Hungarian");
	rc.p.rb.onClick = rc.p.rb1.onClick = OnClickRadioButton;

	rc.p.g = rc.p.add("group");
	rc.p.g.orientation = "row";
	
	rc.p.g.cb = rc.p.g.add("checkbox", undefined, "Print");
	rc.p.g.cb.value = set.hungarian.cbPrint;
	rc.p.g.cb1 = rc.p.g.add("checkbox", undefined, "Online");
	rc.p.g.cb1.value = set.hungarian.cbOnline;	
	rc.p.g.cb2 = rc.p.g.add("checkbox", undefined, "No prices");
	rc.p.g.cb2.value = set.hungarian.cbNoprices;
	//====================================================================================

	// Select folder
	rc.p1 = rc.add("panel", undefined, "Output folder:");
	rc.p1.alignment = "fill";
	rc.p1.st = rc.p1.add("statictext");
	if (outputFolder == null || !outputFolder.exists) { 
		rc.p1.st.text = "No folder has been selected";
	}
	else {
		rc.p1.st.text = TrimPath(outputFolder.absoluteURI);
		rc.p1.st.helpTip = outputFolder.fsName;
	}
	rc.p1.bt = rc.p1.add("button", undefined, "Select...");
	rc.p1.bt.onClick = function() {
		outputFolder = SelectFolder(this, "Pick a folder for output.");
	}

	rc.p2 = rc.add("panel", undefined, "Logging:");
	rc.p2.alignment = "fill";
	rc.p2.cb = rc.p2.add("checkbox", undefined, "Make output overset report");
	rc.p2.cb.alignment = "fill";
	rc.p2.cb.value = set.outputOversetReportFile;
	rc.p2.cb.helpTip = "Make the file in which to store the overset report.";
	
	rc.st = rc.add("statictext", undefined, "", {multiline: true});
	rc.st.graphics.foregroundColor = rc.st.graphics.newPen(rc.st.graphics.PenType.SOLID_COLOR, [1, 0, 0], 1);
	rc.st.alignment = "fill";
	
	// Make settings depending on which radio button was selected last time
	if (set.languageRb == 0) { // Main languages
		rc.p.rb.value = true;
		rc.p.rb1.value = false;
		rc.p.g.enabled = false; // Hungarian check boxes
		lc.enabled = true;
		rc.p2.cb.enabled = true; // 'Make output overset report' check box
		rc.st.text = "Turn PREVIEW OFF";
	}
	else { // Hungarian
		rc.p.rb.value = false;
		rc.p.rb1.value = true;
		rc.p.g.enabled = true; // Hungarian check boxes
		lc.enabled = false;
		rc.p2.cb.enabled = false; // 'Make output overset report' check box
		rc.st.text = "Open DATA MERGE panel, turn PREVIEW ON with the 3RD RECORD. Before export, use 'Hungarian prices' script if necessary.";		
	}

	// Buttons
	rc.bts = rc.add("group");
	rc.bts.orientation = "row";
	rc.bts.alignment = "center";
	rc.bts.ok = rc.bts.add("button", undefined, "OK", {name:"ok"});
	rc.bts.ok.onClick = function() {
		if (rc.p.rb.value && // Make sure at least one check box is selected
			(!lc.p.cb.value && !lc.p.cb1.value && !lc.p.cb2.value && /* czech */
			!lc.p1.cb.value && !lc.p1.cb1.value && !lc.p1.cb2.value && /* english */
			!lc.p2.cb.value && !lc.p2.cb1.value && !lc.p2.cb2.value && /* lithuanian */
			!lc.p3.cb.value && !lc.p3.cb1.value && !lc.p3.cb2.value && /* latvian */
			!lc.p4.cb.value && !lc.p4.cb1.value && !lc.p4.cb2.value && /* romanian */
			!lc.p5.cb.value && !lc.p5.cb1.value && !lc.p5.cb2.value) /* slovakian */
		) {
			alert("At least one check box should be selected in the 'Main languages' section.", scriptName, true);
			return;
		}
		else if (rc.p.rb1.value && (!rc.p.g.cb.value && !rc.p.g.cb1.value && !rc.p.g.cb2.value)) { // Hungarian
			alert("At least one check box should be selected in the 'Hungarian' section.", scriptName, true);
			return;
		} // Make sure the folder is selected
		else if (rc.p1.children[0].text == "No folder has been selected" || outputFolder == null) {
			alert("No 'Output' folder has been selected.", scriptName, true);
			return;			
		}
		else {
			w.close(1);
		}
	}
	rc.bts.cancel = rc.bts.add("button", undefined, "Cancel", {name:"cancel"});
	
	var showDialog = w.show();
	
	if (showDialog == 1) {
		with (set) {
			if (rc.p.rb.value) {
				languageRb = 0;
			}
			else {
				languageRb = 1;
			}
			
			with (czech) {
				cbPrint = lc.p.cb.value;
				cbOnline = lc.p.cb1.value;
				cbNoprices = lc.p.cb2.value;
			}

			with (english) {
				cbPrint = lc.p1.cb.value;
				cbOnline = lc.p1.cb1.value;
				cbNoprices = lc.p1.cb2.value;
			}

			with (lithuanian) {
				cbPrint = lc.p2.cb.value;
				cbOnline = lc.p2.cb1.value;
				cbNoprices = lc.p2.cb2.value;
			}

			with (latvian) {
				cbPrint = lc.p3.cb.value;
				cbOnline = lc.p3.cb1.value;
				cbNoprices = lc.p3.cb2.value;
			}

			with (romanian) {
				cbPrint = lc.p4.cb.value;
				cbOnline = lc.p4.cb1.value;
				cbNoprices = lc.p4.cb2.value;
			}

			with (slovakian) {
				cbPrint = lc.p5.cb.value;
				cbOnline = lc.p5.cb1.value;
				cbNoprices = lc.p5.cb2.value;
			}
		
			with (hungarian) {
				cbPrint = rc.p.g.cb.value;
				cbOnline = rc.p.g.cb1.value;
				cbNoprices = rc.p.g.cb2.value;
			}

			outputOversetReportFile = rc.p2.cb.value;
			outputFolderPath = (outputFolder != null) ? outputFolder.absoluteURI : "";
		}
		
		app.insertLabel("Kas_" + scriptName, set.toSource());
		Main();
	}
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function OnClickRadioButton() {
	if (this.text == "Hungarian") {
		if (debugMode) $.writeln("Clicked Hungarian");
		this.parent.parent.children[3].text = "Open DATA MERGE panel, turn PREVIEW ON with the 3RD RECORD. Before export, use 'Hungarian prices' script if necessary."; // StaticText in red
		this.parent.parent.parent.children[0].enabled = false; // Disable left column
		this.parent.parent.children[2].children[0].enabled = false; // 'Make output overset report' check box
		this.parent.children[2].enabled = true; // Hungarian check boxes
	}
	else {
		if (debugMode) $.writeln("Clicked 'Main languages'");
		this.parent.parent.children[3].text = "Turn PREVIEW OFF"; // StaticText in red
		this.parent.parent.parent.children[0].enabled = true; // Enable left column
		this.parent.parent.children[2].children[0].enabled = true; // 'Make output overset report' check box
		this.parent.children[2].enabled = false; // Hungarian check boxes
	}
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function SelectFolder(button, prompt) {
	var folder = Folder.selectDialog(prompt);
	
	if (folder != null) {
		var panel = button.parent;
		var window = panel.parent;
		var children = panel.children;
		var staticText = children[0];
		var button = button;
		panel.remove(staticText);
		panel.remove(button);
		staticText = panel.add("statictext", undefined, TrimPath(folder.absoluteURI));
		staticText.helpTip = folder.absoluteURI;
		button = panel.add("button", undefined, "Select...");
		button.onClick = function() {
			SelectFolder(this);
		}
		window.layout.layout(true);
		return folder;		
	}
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function GetDialogSettings() {
	set = eval(app.extractLabel("Kas_" + scriptName));
	if (set == undefined) {
		set = {  languageRb: 0,
					czech: {cbPrint: true, cbOnline: true, cbNoprices: true, lang: "CZ"},
					english: {cbPrint: true, cbOnline: true, cbNoprices: true, lang: "EN"},
					lithuanian: {cbPrint: true, cbOnline: true, cbNoprices: true, lang: "LT"},
					latvian: {cbPrint: true, cbOnline: true, cbNoprices: true, lang: "LV"},
					romanian: {cbPrint: true, cbOnline: true, cbNoprices: true, lang: "RO"},
					slovakian: {cbPrint: true, cbOnline: true, cbNoprices: true, lang: "SK"},
					hungarian: {cbPrint: true, cbOnline: true, cbNoprices: true, lang: "HU"},
					outputOversetReportFile: false,
					outputFolderPath: ""
				};
	}

	outputFolder = new Folder(set.outputFolderPath);
	if (!outputFolder.exists) {
		outputFolder = null;
	}

	return set;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function TrimPath(path) {
	var theFile = new File(path);
	if (File.fs == "Macintosh") {
		var trimPath = "..." + theFile.fsName.split("/").splice(-3).join("/");
	}
	else if (File.fs == "Windows" ) {
		var trimPath = ((theFile.fullName.split("/").length > 3) ? ".../" : "") + theFile.fullName.split("/").splice(-3).join("/");
	}
	return trimPath;
}	
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function PreCheck() {
	try {
		if (app.documents.length == 0) ErrorExit("Please open a document and try again.", true);
		doc = app.activeDocument;
		if (doc.converted) ErrorExit("The current document has been modified by being converted from older version of InDesign. Please save the document and try again.", true);
		if (!doc.saved) ErrorExit("The current document has not been saved since it was created. Please save the document and try again.", true);

		// Check pdf presets
		pdfExportPresetOnline = app.pdfExportPresets.itemByName("Dedra gazetka ONLINE"); // global
		if (!pdfExportPresetOnline.isValid) ErrorExit("'Dedra gazetka ONLINE' pdf preset is not installed.", true);
		pdfExportPresetPrint = app.pdfExportPresets.itemByName("Dedra gazetka DRUK"); // global
		if (!pdfExportPresetPrint.isValid) ErrorExit("'Dedra gazetka DRUK' pdf preset is not installed.", true);

		// Check data merge
		dataMerge = doc.dataMergeProperties; // global
		dataMergePrefs = dataMerge.dataMergePreferences; // global
		var recordRangeNum = Number(dataMergePrefs.recordRange.split("-")[1]);

		if (dataMerge.dataMergeFields.length == 0) {
			ErrorExit("No Data Source has been selected.", true);
		}

		CreateDialog();
	}
	catch(err) {
		if (debugMode) {
			$.writeln(err.message + ", line: " + err.line);
		}
		else {
			alert("ERROR: " + err.message + ", line: " + err.line, scriptName, true);
		}
	}
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function GetDuration(startTime, endTime) {
	var str;
	var duration = (endTime - startTime)/1000;
	duration = Math.round(duration);
	if (duration >= 60) {
		var minutes = Math.floor(duration/60);
		var seconds = duration - (minutes * 60);
		str = minutes + ((minutes != 1) ? " minutes, " :  " minute, ") + seconds + ((seconds != 1) ? " seconds" : " second");
		if (minutes >= 60) {
			var hours = Math.floor(minutes/60);
			minutes = minutes - (hours * 60);
			str = hours + ((hours != 1) ? " hours, " : " hour, ") + minutes + ((minutes != 1) ? " minutes, " :  " minute, ") + seconds + ((seconds != 1) ? " seconds" : " second");
		}
	}
	else {
		str = duration + ((duration != 1) ? " seconds" : " second");
	}

	return str;
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function ErrorExit(error, icon) {
	alert(error, scriptName, icon);
	exit();
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------
function GetActiveScript() {
    try {
        return app.activeScript;
    }
    catch(err) {
        return new File(err.fileName);
    }
}
//--------------------------------------------------------------------------------------------------------------------------------------------------------